@@ -200,8 +200,14 @@ def overload(
     from numba.cuda.typing.templates import make_overload_template, infer_global
 
     # set default options
+    if jit_options is not None and "inline" in jit_options:
+        raise ValueError(
+            "`inline` should be passed as a separate argument, not in `jit_options`"
+        )
     opts = _overload_default_jit_options.copy()
     opts.update(jit_options or {})  # let user options override
+    if inline is not None:
+        opts["inline"] = inline
 
     # TODO: abort now if the kwarg 'target' relates to an unregistered target,
     # this requires sorting out the circular imports first.
